@@ -296,28 +296,29 @@ class gui:
         self.scores = None
 
     def threshold_and_nms(self, thr, nms_thr) -> Tensor:
+        if self.all_boxes is not None:
+            ind = self.all_scores > (thr / 100)
 
-        ind = self.all_scores > (thr / 100)
-        if isinstance(self.all_labels, list):
-            _labels = torch.tensor([1 if l == 'OHC' else 2 for l, i in zip(self.all_labels, ind) if i > 0])
-        else:
-            _labels = self.all_labels[ind]
+            if isinstance(self.all_labels, list):
+                _labels = torch.tensor([1 if l == 'OHC' else 2 for l, i in zip(self.all_labels, ind) if i > 0])
+            else:
+                _labels = self.all_labels[ind]
 
-        _boxes = self.all_boxes[ind, :]
-        _scores = self.all_scores[ind]
+            _boxes = self.all_boxes[ind, :]
+            _scores = self.all_scores[ind]
 
-        ind = torchvision.ops.nms(_boxes, _scores, nms_thr / 100)  # int indicies
+            ind = torchvision.ops.nms(_boxes, _scores, nms_thr / 100)  # int indicies
 
-        _boxes = _boxes[ind, :]
-        _scores = _scores[ind]
+            _boxes = _boxes[ind, :]
+            _scores = _scores[ind]
 
-        _labels = ['OHC' if _labels[i] == 1 else 'IHC' for i in ind]
+            _labels = ['OHC' if _labels[i] == 1 else 'IHC' for i in ind]
 
-        self.boxes = _boxes
-        self.labels = _labels
-        self.scores = _scores
+            self.boxes = _boxes
+            self.labels = _labels
+            self.scores = _scores
 
-        self.update_cell_counts()
+            self.update_cell_counts()
 
     def get_color_histogram(self):
         color = ['r', 'g', 'b']
@@ -479,9 +480,6 @@ class gui:
             sg.popup_quick_message('Loading model from file. May take a while.')
             self.model = FasterRCNN_from_url(url=__model_url__, device=self.device)
 
-            _model = torch.jit.script(self.model)
-            _model.save('/home/chris/Desktop/frcnn.trch')
-
         _image = self.scaled_image.clone()[0:3,...]
 
         for i in range(3):
